import argparse
import logging
import csv

from scrapy.crawler import CrawlerProcess
from volleystats.spiders.match import HomeStatsSpider, GuestStatsSpider
from volleystats.spiders.competition import CompetitionMatchesSpider
from volleystats.version import __version__

version = __version__
welcome_msg = f'''
                    .
                    |`.
                    |  `.
                    |-_  `.
                    |  -_  `._
____________________|____-_ _|_______________,
',                         -_|                ',
  ',                         |                  ',
    ',                       |                    ',
      ',_____________________|______________________', v{version}
'''

started_msg = '\nvolleystats: started'
finished_msg = 'volleystats: finished'

def main():
	print(welcome_msg)

	parser = argparse.ArgumentParser(
		prog='volleystats',
		description='Command-line tool to scrape volleyball statistics from Data Project Web Competition websites',
		epilog='Found a bug? https://github.com/claromes/volleystats/issues'
	)

	parser.add_argument(
		'--fed', '-f',
		dest='fed',
		required=True,
		help='Federation acronym: <Fed_Acronym>-web.dataproject.com'
	)

	group = parser.add_mutually_exclusive_group(required=True)

	group.add_argument(
		'--match', '-m',
		dest='match',
		type=int,
		help='ID of the match: <Fed_Acronym>-web.dataproject.com/MatchStatistics?mID=<Match_ID>'
	)

	group.add_argument(
		'--comp', '-c',
		dest='comp',
		type=int,
		help='ID of the competition: <Fed_Acronym>-web.dataproject.com/CompetitionMatches?ID=<Competition_ID>'
	)

	group.add_argument(
		'--batch', '-b',
		dest='batch',
		type=str,
		help='CSV file path with Match IDs (Competition Matches output): data/<Fed_Acronym>-<Competition_ID>-<start_year>-<end_year>-competition-matches.csv'
	)

	parser.add_argument(
		'--pid', '-p',
		dest='pid',
		type=int,
		required=False,
		help='PID of the competition: <Fed_Acronym>-web.dataproject.com/CompetitionMatches?ID=<Competition_ID>&PID=<Competition_PID>'
	)

	parser.add_argument(
		'--log', '-l',
		dest='log',
		action='store_true',
		required=False,
		help='View the logging during scraping'
	)

	args = vars(parser.parse_args())

	print(started_msg)

	logging.disable(logging.CRITICAL)

	if args['log']:
		logging.disable(logging.NOTSET)

	if args['match']:
		fed_acronym = args['fed']
		match_id = args['match']

		match_process = CrawlerProcess(settings={
			'FEEDS': {
				'data/%(fed_acronym)s-%(match_id)s-%(name)s.csv': {
				'format': 'csv',
				'overwrite': True
				},
			},
		})

		match_process.crawl(HomeStatsSpider, fed_acronym=fed_acronym, match_id=match_id)
		match_process.crawl(GuestStatsSpider, fed_acronym=fed_acronym, match_id=match_id)
		match_process.start()

		print(finished_msg)
	elif args['comp'] or (args['comp'] and args['pid']):
		fed_acronym = args['fed']
		competition_id = args['comp']
		competition_pid = args['pid']

		comp_process = CrawlerProcess(settings={
			'FEEDS': {
				'data/%(fed_acronym)s-%(competition_id)s-%(competition_pid)s-%(name)s.csv': {
				'format': 'csv',
				'overwrite': True
				},
			},
		})

		comp_process.crawl(CompetitionMatchesSpider, fed_acronym=fed_acronym, competition_id=competition_id, competition_pid=competition_pid)
		comp_process.start()

		print(finished_msg)
	elif args['batch']:
		fed_acronym = args['fed']
		csv_file_path = args['batch']

		match_ids = []

		with open(csv_file_path, 'r') as f:
			csv_reader = csv.DictReader(f)
			for row in csv_reader:
				match_ids.append(row['Match ID'])

		batch_process = CrawlerProcess(settings={
			'FEEDS': {
				'data/%(fed_acronym)s-%(match_id)s-%(name)s.csv': {
				'format': 'csv',
				'overwrite': True
				},
			},
		})

		for match_id in match_ids:
			match_id_started_msg = f'\nvolleystats: starting {match_id}...'
			print(match_id_started_msg)

			batch_process.crawl(HomeStatsSpider, fed_acronym=fed_acronym, match_id=match_id)
			batch_process.crawl(GuestStatsSpider, fed_acronym=fed_acronym, match_id=match_id)

		batch_process.start()
		print(finished_msg)

if __name__ == '__main__':
    main()



import scrapy
import os
import re

from ..utils import *

class HomeStatsSpider(scrapy.Spider):
    name = 'home_stats'

    def __init__(self, fed_acronym='', match_id='', **kwargs):
        self.start_urls = [f'https://{fed_acronym}-web.dataproject.com/MatchStatistics.aspx?mID={match_id}']
        self.fed_acronym = fed_acronym
        self.match_id = match_id
        match_date = ''
        home_team = ''
        home_team_file = ''

        super().__init__(**kwargs)

    def start_requests(self):        
        cookies = {f'CompetitionLangCode{self.fed_acronym}': 'en-GB'}
        yield scrapy.Request(self.start_urls[0], cookies=cookies, callback=self.parse)

    def parse(self, response):
        match_date_text = response.xpath("normalize-space(//span[@id='Content_Main_LB_DateTime']/text())").get()
        enGB = response.xpath("//*[contains(@class, 'RCB_Culture_en-GB')]/span/input/@value").get()

        if enGB == 'EN':
            match_date = parse_engb_date(match_date_text)

        home_team_string = response.xpath("normalize-space(//span[@id='Content_Main_LBL_HomeTeam']/text())").get()
        if home_team_string:
            home_team = home_team_string.lower()
            home_team_file = re.sub(' ', '', home_team)

        coach = response.xpath("//span[@id='Content_Main_ctl17_RP_MatchStats_Coach_Home_0']/text()").get()
        if coach:
            coach = coach.lower()
            coach = parse_coach(coach)

        location = response.xpath("//span[@id='Content_Main_LB_Stadium']/text()").get()
        if location:
            location = location.lower()

        home_players = response.xpath("//div[@id='Content_Main_ctl17_RP_MatchStats_RPL_MatchStats_0']/div[3]/div/div/table/tbody/tr")

        for player in home_players:
            player_number = player.xpath('./td[1]/p/span/text()').get()
            player_name = player.xpath('./td[2]/p/span/b/text()').get()
            if player_name:
                player_name = player_name.lower()

            points_tot = player.xpath('./td[8]/p/span/text()').get()
            points_BP = player.xpath('./td[9]/p/span/text()').get()
            points_WL = player.xpath('./td[10]/p/span/text()').get()

            serve_tot = player.xpath('./td[12]/p/span/text()').get()
            serve_err = player.xpath('./td[13]/p/span/text()').get()
            serve_ace = player.xpath('./td[14]/p/span/text()').get()

            reception_tot = player.xpath('./td[15]/p/span/text()').get()
            reception_err = player.xpath('./td[16]/p/span/text()').get()
            reception_pos = player.xpath('./td[17]/p/span/text()').get()
            reception_exec = player.xpath('./td[18]/p/span/text()').get()

            attack_tot = player.xpath('./td[21]/p/span/text()').get()
            attack_err = player.xpath('./td[22]/p/span/text()').get()
            attack_block = player.xpath('./td[23]/p/span/text()').get()
            attack_exc = player.xpath('./td[24]/p/span/text()').get()
            attack_exc_perc = player.xpath('./td[25]/p/span/text()').get()

            block_points = player.xpath('./td[27]/p/span/text()').get()

            yield {
                'Match ID': self.match_id,
                'Match Date': match_date,
                'Home Team': home_team,
                'Home Coach': coach,
                'Stadium': location,
                'Number': player_number,
                'Name': player_name,
                'Total Points': points_tot,
                'Break Points': points_BP,
                'W-L': points_WL,
                'Total Serve': serve_tot,
                'Serve Errors': serve_err,
                'Ace': serve_ace,
                'Total Receptions': reception_tot,
                'Reception Erros': reception_err,
                'Positive Pass Percentage': reception_pos,
                'Excellent/ Perfect Pass Percentage': reception_exec,
                'Total Attacks': attack_tot,
                'Attack Erros': attack_err,
                'Blocked Attack': attack_block,
                'Attack Points (Exc.)': attack_exc,
                'Attack Points Percentage (Exc.%)': attack_exc_perc,
                'Block Points': block_points
            }

        self.match_date = match_date
        self.home_team = home_team
        self.home_team_file = home_team_file

    def closed(spider, reason):
        src = f'data/{spider.fed_acronym}-{spider.match_id}-home_stats.csv'
        dst = f'data/{spider.fed_acronym}-{spider.match_id}-{spider.match_date}-home-{spider.home_team_file}.csv'

        try:
            os.rename(src, dst)

            print(f'volleystats: {dst} file was created')
        except(FileExistsError):
            print(f'volleystats: file {dst} already exists.\n{src} was created or renamed')

class GuestStatsSpider(scrapy.Spider):
    name = 'guest_stats'

    def __init__(self, fed_acronym='', match_id='', **kwargs):
        self.start_urls = [f'https://{fed_acronym}-web.dataproject.com/MatchStatistics.aspx?mID={match_id}']
        self.fed_acronym = fed_acronym
        self.match_id = match_id
        match_date = ''
        guest_team = ''
        guest_team_file = ''

        super().__init__(**kwargs)

    def start_requests(self):        
        cookies = {f'CompetitionLangCode{self.fed_acronym}': 'en-GB'}
        yield scrapy.Request(self.start_urls[0], cookies=cookies, callback=self.parse)

    def parse(self, response):
        match_date_text = response.xpath("normalize-space(//span[@id='Content_Main_LB_DateTime']/text())").get()
        enGB = response.xpath("//*[contains(@class, 'RCB_Culture_en-GB')]/span/input/@value").get()

        if enGB == 'EN':
            match_date = parse_engb_date(match_date_text)

        guest_team_string = response.xpath("normalize-space(//span[@id='Content_Main_LBL_GuestTeam']/text())").get()
        if guest_team_string:
            guest_team = guest_team_string.lower()
            guest_team_file = re.sub(' ', '', guest_team)

        coach = response.xpath("//span[@id='Content_Main_ctl17_RP_MatchStats_Coach_Guest_0']/text()").get()
        if coach:
            coach = coach.lower()
            coach = parse_coach(coach)

        location = response.xpath("//span[@id='Content_Main_LB_Stadium']/text()").get()
        if location:
            location = location.lower()

        guest_players = response.xpath("//div[@id='Content_Main_ctl17_RP_MatchStats_RPL_MatchStats_0']/div[5]/div/div/table/tbody/tr")

        for player in guest_players:
            player_number = player.xpath('./td[1]/p/span/text()').get()
            player_name = player.xpath('./td[2]/p/span/b/text()').get()
            if player_name:
                player_name = player_name.lower()

            points_tot = player.xpath('./td[8]/p/span/text()').get()
            points_BP = player.xpath('./td[9]/p/span/text()').get()
            points_WL = player.xpath('./td[10]/p/span/text()').get()

            serve_tot = player.xpath('./td[12]/p/span/text()').get()
            serve_err = player.xpath('./td[13]/p/span/text()').get()
            serve_ace = player.xpath('./td[14]/p/span/text()').get()

            reception_tot = player.xpath('./td[15]/p/span/text()').get()
            reception_err = player.xpath('./td[16]/p/span/text()').get()
            reception_pos = player.xpath('./td[17]/p/span/text()').get()
            reception_exc = player.xpath('./td[18]/p/span/text()').get()

            attack_tot = player.xpath('./td[21]/p/span/text()').get()
            attack_err = player.xpath('./td[22]/p/span/text()').get()
            attack_block = player.xpath('./td[23]/p/span/text()').get()
            attack_exc = player.xpath('./td[24]/p/span/text()').get()
            attack_exc_perc = player.xpath('./td[25]/p/span/text()').get()

            block_points = player.xpath('./td[27]/p/span/text()').get()

            yield {
                'Match ID': self.match_id,
                'Match Date': match_date,
                'Guest Team': guest_team,
                'Guest Coach': coach,
                'Stadium': location,
                'Number': player_number,
                'Name': player_name,
                'Total Points': points_tot,
                'Break Points': points_BP,
                'W-L': points_WL,
                'Total Serve': serve_tot,
                'Serve Errors': serve_err,
                'Ace': serve_ace,
                'Total Receptions': reception_tot,
                'Reception Erros': reception_err,
                'Positive Pass Percentage (Pos%)': reception_pos,
                'Excellent/ Perfect Pass Percentage (Exc.%)': reception_exc,
                'Total Attacks': attack_tot,
                'Attack Erros': attack_err,
                'Blocked Attack': attack_block,
                'Attack Points (Exc.)': attack_exc,
                'Attack Points Percentage (Exc.%)': attack_exc_perc,
                'Block Points': block_points
            }

        self.match_date = match_date
        self.guest_team = guest_team
        self.guest_team_file = guest_team_file

    def closed(spider, reason):
        src = f'data/{spider.fed_acronym}-{spider.match_id}-guest_stats.csv'
        dst = f'data/{spider.fed_acronym}-{spider.match_id}-{spider.match_date}-guest-{spider.guest_team_file}.csv'

        try:
            os.rename(src, dst)

            print(f'volleystats: {dst} file was created')
        except(FileExistsError):
            print(f'volleystats: file {dst} already exists.\n{src} was created or renamed')



import scrapy
import os
import re

from ..utils import *

class CompetitionMatchesSpider(scrapy.Spider):
    name = 'competition_matches'

    def __init__(self, fed_acronym='', competition_id='', competition_pid='', **kwargs):
        self.start_urls = [f'https://{fed_acronym}-web.dataproject.com/CompetitionMatches.aspx?ID={competition_id}&PID={competition_pid}']
        self.competition_id = competition_id
        self.competition_pid = competition_pid
        self.fed_acronym = fed_acronym
        match_id = ''
        match_date = ''
        match_location = ''
        home_team = ''
        home_points = ''
        guest_team = ''
        guest_points = ''
        self.first_item_date = ''
        self.last_item_date = ''

        super().__init__(**kwargs)

    def start_requests(self):        
        cookies = {f'CompetitionLangCode{self.fed_acronym}': 'en-GB'}
        yield scrapy.Request(self.start_urls[0], cookies=cookies, callback=self.parse)

    def parse(self, response):
        competition_items = []

        matches = response.xpath("//div[@id='printableArea']/div/div/div/div/div[position() >= 1]/div[2]/div")

        for match in matches:
            match_id_string = match.xpath("./div/div/div[5]/p/@onclick").get()
            match_id = re.search(r'mID=(\d+)', match_id_string).group(1)

            match_date_text = match.xpath("./div/div/div/p[1]/span[1]/text()").get()
            match_date = parse_short_date(match_date_text)

            match_location = match.xpath("./div/div/div/p[2]/span[1]/text()").get()
            if match_location:
                match_location = match_location.lower()

            home_team = match.xpath("./div/div/div[5]/p/span/*/text() | ./div/div/div[5]/p/span/text()").get()
            if home_team:
                home_team = home_team.lower()

            home_points = match.xpath("./div/div/div[7]/p[1]/span[1]/b/text()").get()

            guest_team = match.xpath("./div/div/div[9]/p/span/*/text() | ./div/div/div[9]/p/span/text()").get()
            if guest_team:
                guest_team = guest_team.lower()

            guest_points = match.xpath("./div/div/div[7]/p[1]/span[3]/b/text()").get()

            competition = {
                'Match ID': match_id,
                'Match Date': match_date,
                'Stadium': match_location,
                'Home Team': home_team,
                'Home Points': home_points,
                'Guest Team': guest_team,
                'Guest Points': guest_points,
            }

            competition_items.append(competition)
            yield competition

        first_date = competition_items[0]['Match Date']
        last_date = competition_items[-1]['Match Date']

        regex = r"\b\d{4}\b"
        match_start = re.search(regex, first_date)
        match_final = re.search(regex, last_date)

        if match_start:
            self.first_item_date = match_start.group()

        if match_final:
             self.last_item_date = match_final.group()
        
    def closed(spider, reason):
        src = f'data/{spider.fed_acronym}-{spider.competition_id}-{spider.competition_pid}-competition_matches.csv'

        if spider.competition_pid != None:
            spider.competition_pid = f'{spider.competition_pid}-'

        if spider.competition_pid == None:
            spider.competition_pid = ''


        dst = f'data/{spider.fed_acronym}-{spider.competition_id}-{spider.competition_pid}{spider.first_item_date}-{spider.last_item_date}-competition-matches.csv'

        try:
            os.rename(src, dst)

            print(f'volleystats: {dst} file was created')
        except(FileExistsError):
            print(f'volleystats: file {dst} already exists.\n{src} was created or renamed')


from setuptools import setup, find_packages
import re

with open('README.md', 'r', encoding='utf-8') as f:
    long_description = f.read()

with open('requirements.txt', 'r', encoding='utf-8') as f:
    install_requires = f.read().splitlines()

with open('volleystats/version.py', 'r', encoding='utf-8') as f:
    version = re.search(r"^__version__\s*=\s*'(.*)'.*$",
        f.read(), flags=re.MULTILINE).group(1)

setup(
    name='volleystats',
    version=version,
    author='Claromes',
    author_email='support@claromes.com',
    description='Command-line tool to scrape volleyball statistics from Data Project Web Competition websites',
    license='GPLv3',
    long_description=long_description,
    long_description_content_type='text/markdown',
    keywords='volleyball sports command-line datasets analytics',
    url='https://github.com/claromes/volleystats',
    project_urls={
        'Documentation': 'https://github.com/claromes/volleystats#documentation',
        'Releases': 'https://github.com/claromes/volleystats/releases',
        'Bug Tracker': 'https://github.com/claromes/volleystats/issues'
    },
    packages=find_packages(exclude=['docs']),
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Framework :: Scrapy',
        'Intended Audience :: Science/Research',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.8',
        'Topic :: Scientific/Engineering',
        'Topic :: Scientific/Engineering :: Information Analysis',
        'Topic :: Utilities'
    ],
    python_requires='>=3.8',
    install_requires=install_requires,
    entry_points = {
		'console_scripts': [
			'volleystats = volleystats.main:main',
		],
	},
)



import re
from datetime import datetime

# '28/10/2022 - 19:30' or '28.10.2022 - 19:30' to 2022-10-28
def parse_short_date(short_date_string):
    if '/' in short_date_string:
        date_format = '%d/%m/%Y'
    elif '.' in short_date_string:
        date_format = '%d.%m.%Y'

    short_date_obj = datetime.strptime(short_date_string, f'{date_format} - %H:%M')
    parsed_short_date = short_date_obj.strftime('%Y-%m-%d')

    return parsed_short_date

# en-GB: '05 November 2022 - 21:30' to 2022-11-05
def parse_engb_date(date_string):
    str_1 = date_string.replace(' ', '-')
    str_2 = str_1.rsplit('---', 1)[0]

    parsed_engb_date = datetime.strptime(str_2, '%d-%B-%Y').date()

    return parsed_engb_date

# (coach: schimtz guilherme) to schimtz guilherme
def parse_coach(coach_name):
    regex = re.compile(r'\(coach: |\)')
    coach = regex.sub('', coach_name)

    return coach